
export default class TorqueCanvas {
    constructor(options) {
        this.options = Object.assign({
            padding: 30
        }, options);
        this.canvas = this._createCanvas(this.options.el);
        
        this.canvas.translate(this.options.padding, this.options.padding);

        this.getDataRenderLine(this.options.vertical, this.options.horizontal)
            .renderCoordinate();   

        // 测试一个矩形
        this._drawRectInWord({x:100, y:100}, 100, 40, 'hello');
        this._drawRectInWord({x:200, y:200}, 100, 40, 'world', {fillStyle: 'red'});
        // 测试一个线性
        this._drawLine({x:20, y: 30}, {x: 100, y: 300}, 'line', {strokeStyle: 'blue', lineWidth: 6});
    }
    /**
     * 创建canvas画布对象
     * @param {*} el 
     * @return canvas
     */
    _createCanvas(el) {
        if(el && el.nodeType == 1) {
            return el.getContext('2d');
        }
    }
    
    /**
     * 重新渲染数据
     * @param {Object} options={horizontal=[], vertical=[]} 
     */
    initialRender(options={}) {
        this.getDataRenderLine(options.vertical, options.horizontal)
    }

    /**
     * 根据数据渲染虚线
     * @param {Array} vertical:垂直坐标(从大到小)
     * @param {Array} horizontal: 水平坐标 
     */
    getDataRenderLine(vertical=[], horizontal=[]) {
        var vertical = this.units_arraySortDown(vertical);
        var horizontal = this.units_arraySortUp(horizontal);
        const {verticalSize, horizontalSize, width, height} = this.getDataCalcCellSize(vertical, horizontal);
        
        // 垂直
        vertical.forEach((v, index) => {
            let start = {x: 0, y: height * index};
            let end = {x: horizontalSize, y: height * index};
            this._drawLine(start, end);
            this._drawText(v, start.x - 15, start.y + 5);
        });

        horizontal.forEach((h, index) => {
            let start = {x: width * index, y: 0};
            let end = {x: width * index, y: verticalSize};
            this._drawLine(start, end); 
            this._drawText(h, start.x, verticalSize + 5);
        });

        return this;
    }
    

    /**
     * 根据数据计算单位长和宽
     * @param {Array} vertial: 垂直的数据
     * @param {Array} horizontal: 水平的数据
     * @return {Object}
     */
    getDataCalcCellSize(vertical=[], horizontal=[]) {
        var width = Math.ceil((this.options.el.width - this.options.padding*2) / horizontal.length), 
            height = Math.ceil((this.options.el.height - this.options.padding*2) / vertical.length);

        return {
            width,
            height,
            horizontalSize: (horizontal.length) * width,
            verticalSize: (vertical.length) * height
        }
    }

    /**
     * 初始化坐标系--one step
     */
    renderCoordinate() {
        var coordinate_xdistance = this.options.el.width - this.options.padding * 2;
        var coordinate_ydistance = this.options.el.height - this.options.padding * 2;
        
        // y轴
        this._drawLine({x: 0, y: 0}, {x: 0, y: coordinate_ydistance}, 'line', {
            strokeStyle: 'red'
        });
        // x轴
        this._drawLine({x: 0, y: coordinate_ydistance}, {x: coordinate_xdistance, y: coordinate_ydistance}, 'line', {
            strokeStyle: 'red'
        });
        return this;
    }

    /**
     * 画线
     * @param {Object} start={x, y} 开始的点坐标
     * @param {Object} end={x, y} 结束的点坐标
     * @param {String} type: 虚线和实线
     * @param {Object} options: 配置项
     */
    _drawLine(start, end, type='dash', options={}) {
        this.canvas.lineWidth = options.lineWidth || 1;
        this.canvas.strokeStyle = options.strokeStyle || '#303431';

        this.canvas.beginPath();
        this.canvas.moveTo(start.x, start.y);
        this.canvas.lineTo(end.x, end.y);
        //type == 'dash' ? this.canvas.setLineDash([10, 3]) : null;
        this.canvas.stroke();
        this.canvas.closePath();
    }

    /**
     * 绘制文本
     * @param {String} text 
     * @param {*} x 
     * @param {*} y 
     */
    _drawText(text, x, y) {
        this.canvas.fillStyle = 'black';
        this.canvas.textAlign = 'center';
        this.canvas.textBaseline = 'middle';
        this.canvas.font = '24px serif';
        this.canvas.fillText(text, x, y);
    }

    /**
     * 绘制带有文字的矩形
     * @param {Object} start={x, y} 开始坐标 
     * @param {*} width 
     * @param {*} height 
     * @param {Object} options:配置项
     */
    _drawRectInWord(start, width, height, text, options={}) {
        const {x, y} = start;

        this.canvas.fillStyle = options.fillStyle || '#ccc';
        this.canvas.fillRect(x, y, width, height);

        this._drawText(text, x+width/2, y+height/2);
    }

    /**
     * 数组从小到大排序
     * @param {Array} array 
     */
    units_arraySortUp(array) {
        return array.sort( (a, b) => a - b);
    }

    /**
     * 数组从大到小排序
     * @param {Array} array 
     */
    units_arraySortDown(array) {
        return array.sort((a, b) => b - a);
    }
}